<?php

namespace Drupal\tome_aws;

use Aws\S3\S3Client;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\File\FileSystemInterface;
use Drupal\tome_base\PathTrait;
use Drupal\tome_static\StaticGeneratorInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;

/**
 * A service to wrap batch operations for deploying to AWS.
 */
class TomeAwsDeployBatch {

  use PathTrait;
  use DependencySerializationTrait;

  /**
   * The static generator.
   *
   * @var \Drupal\tome_static\StaticGeneratorInterface
   */
  protected $static;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * TomeAwsDeployBatch constructor.
   *
   * @param \Drupal\tome_static\StaticGeneratorInterface $static
   *   The static generator.
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   */
  public function __construct(StaticGeneratorInterface $static, Client $http_client, ConfigFactoryInterface $config_factory, FileSystemInterface $file_system) {

    $this->static     = $static;
    $this->httpClient = $http_client;
    $this->config     = $config_factory->get('tome_aws.settings');
    $this->fileSystem = $file_system;
  }

  /**
   * Determines if the site is configured to properly deploy.
   *
   * @return bool
   *   Whether or not the site is configured to properly deploy.
   */
  public function checkConfiguration() {

    return !empty($this->config->get('aws_s3_client_id')) && !empty($this->config->get('aws_s3_client_secret')) && !empty($this->config->get('aws_s3_bucket_name')) && !empty($this->config->get('aws_s3_bucket_prefix'));
  }

  /**
   * Determines if a static build exists.
   *
   * @return bool
   *   Whether or not a static build exists.
   */
  public function checkStaticBuild() {

    return file_exists($this->static->getStaticDirectory());
  }

  /**
   * Gets a batch object for deploying to AWS.
   *
   * @param string $title
   *   The deploy title.
   *
   * @return \Drupal\Core\Batch\BatchBuilder
   *   The batch builder.
   */
  public function getBatch($title) {

    $batch_builder = new BatchBuilder();
    $files         = [];
    foreach ($this->fileSystem->scanDirectory($this->static->getStaticDirectory(), '/.*/') as $file) {
      $files[] = $file->uri;
    }
    foreach (array_chunk($files, 10) as $chunk) {
      $batch_builder->addOperation([$this, 'uploadSyncFiles'], [$chunk]);
    }
    return $batch_builder;
  }

  public function uploadSyncFiles($localFiles, &$context) {

    foreach ($localFiles as $localFile) {

      $relativeFilePath = str_replace($this->static->getStaticDirectory(), '', $localFile);

      $this->uploadSyncFile($localFile, $relativeFilePath);

    }

  }

  private function uploadSyncFile(string $localFile, string $relativeFilePath) {

    $remotePrefix = 'https://s3.eu-central-1.amazonaws.com/' . $this->config->get('aws_s3_bucket_name') . $this->config->get('aws_s3_bucket_prefix');

    if ($this->isDifferent($localFile, $remotePrefix . $relativeFilePath)) {

      $s3 = new S3Client([
        'region'      => 'eu-central-1',
        'version'     => '2006-03-01',
        'credentials' => [
          'key'    => $this->config->get('aws_s3_client_id'),
          'secret' => $this->config->get('aws_s3_client_secret'),
        ],
      ]);

      $s3->putObject([
        'Bucket' => $this->config->get('aws_s3_bucket_name'),
        'Key'    => $this->config->get('aws_s3_bucket_prefix') . $remotePrefix,
        'Body'   => fopen($localFile, 'r'),
        'ACL'    => 'public-read',
      ]);

    }

  }

  private function isDifferent(string $localFile, string $remoteFile) {

    try {

      $remoteFileETag = $this->httpClient->request('HEAD', $remoteFile)->getHeader('ETag');

    } catch(ClientException $clientException) {

      return TRUE;

    }

    $remoteFileETag = reset($remoteFileETag);

    $localFileMd5  = md5_file($localFile);
    $localFileETag = '"' . $localFileMd5 . '"';

    return $localFileETag !== $remoteFileETag;

  }

}
